// ----------------------------------------------------------------------------
// * Raima Database Manager
// *
// * Copyright (c) 2012 Raima Inc.,  All rights reserved.
// *
// * Use of this software, whether in source code format, or in executable,
// * binary object code form, is governed by the Raima LICENSE which
// * is fully described in the LICENSE.TXT file, included within this
// * distribution of files.
// * ----------------------------------------------------------------------------

#include <iostream>

// rdm.h is required in all files using the RDM Database
#include "rdm.h"

// obj05_db.h was generated from the DDL file obj05_db.ddl.
#include "obj05_db.h"

// obj05_db_api.h was generated from the DDL file obj05_db.ddl. with 
// the -cpp option
#include "obj05_db_api.h"

// Name space for the RDM C++ interface
using namespace RDM_CPP;

// EXAMPLE - obj05 for C++
// Each time you run this example:
//   1) The database is initialized (if necessary)
//   2) 5 new person records are added to the database
//   3) A list of each person is displayed in index order.

// The DDL (Database Definition Language) is in obj05_db.ddl. The database
// schema is embedded into the application and the database will be created
// the first time the application is run.  Subsequent runs will use the now
// existing database.

// The following RDM C++ methods are illustrated:
//      Db->Open
//      Db->New_recordWithFieldValues
//      Db->Get_recordsByIndex
//      Cursor->AfterLast
//      Cursor->GetFieldValues
//      Cursor++
// They are fully documented in http://docs.raima.com/rdm/.

// Each of these methods can throw and an exception of type rdm_exception

// The actual database is stored in a directory named 'obj05_db' in this
// project directory.

class obj05
{
public:
    Db_obj05_db   db;
    Cursor_person person_cursor;

    void InsertData () throw (rdm_exception&);
    void DisplayData () throw (rdm_exception&);
    void Initialize () throw (rdm_exception&);
    void Cleanup () throw (rdm_exception&);
};

void obj05::Initialize() throw (rdm_exception&)
{
    int16_t      do_initialize = 0;
    
    // Seed the random number generator
    srand((uint32_t) time(NULL));

    // Open the database and check to see if the db file exists
    try
    {
        db = db.Open("obj05_db", "x");
        person_cursor = db.Get_person_records();
    }
    catch (rdm_exception& e)
    {
        // This is the error code if the data file does not exist.
        // In this case we will call db->Initialize to create the file.
        if (e.error() == S_NOFILE)
        {
            do_initialize = 1;
        }
        else
        {
            cerr << "Error " << e.error() << " (" << e.what() << ") opening obj05_db database" << endl;
            throw e;
        }
    }

    // Create the database files (if necessary)
    if(do_initialize)
    {
        try
        {
            db.Initialize();
        }
        catch (rdm_exception& e)        
        {
            cerr << "Error " << e.error() << " (" << e.what() << ") initializing obj02_db database" << endl;
            throw e;
        }
    }
    return;
}

void obj05::InsertData() throw (rdm_exception&)
{
    struct person person_rec;
    int16_t       ii;
    int32_t       value;

    cout << "Adding 5 new person records" << endl;
    // Add 5 new records to the database
    try
    {
        for (ii = 0; ii < 5; ii++)
        {
            value = rand() % 10000;
            sprintf(person_rec.last_four_ssn, "%04d", value);
            person_cursor = db.New_person_recordWithFieldValues(&person_rec);

            cout << '\t' << person_rec.last_four_ssn << endl;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") adding records" << endl;
        throw (e);
        return;
    }
    return;
}
    
void obj05::DisplayData() throw (rdm_exception&)
{
    struct person person_rec;

    cout << endl << "List all records in index order" << endl;
    try
    {
        // Loop through each person record via the index
        person_cursor = db.Get_person_recordsBy_last_four_ssn();
        while (!person_cursor.AfterLast())
        {
            person_cursor.GetFieldValues(&person_rec);
            cout << '\t' << person_rec.last_four_ssn << endl;

            person_cursor++;
        }
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
    }   
}

void obj05::Cleanup() throw (rdm_exception&)
{
    return;
}

int32_t main(void)
{
    obj05 obj;

    try
    {
        obj.Initialize();
        obj.InsertData();
        obj.DisplayData();
        obj.Cleanup();
    }
    catch (rdm_exception& e)
    {
        cerr << "Error " << e.error() << " (" << e.what() << ") reading records" << endl;
        return EXIT_FAILURE;
    }
    
    cout << endl << "Press <Enter> to continue . . .";
    getchar();
    return EXIT_SUCCESS;
}
